#!/bin/bash

# builds the website, and when PULL_ARTIFACTS=1 is set also
# downloads all binaries and source zip.
# it assumes the parent project does build and run, and
# uses it to get the current version.
# it them assumes this version is available as a tag.

set -ex

project_root="https://git.argent.systems/alice/tumelune"
root="$(dirname "$BASH_SOURCE")"

dest="$root/../target/site"
src="$root/../src"

version=$(cd $root/../ && cargo run -- --version | cut -d" " -f2)

mkdir -p $dest/static/
mkdir -p $dest/files/

cp -v $root/index.css $dest/
cp -v $root/../screenshot.png $dest/

export TL_VERSION="v$version"
python3 $root/render_index.py > $dest/index.html
cp -vr $src/../static/* $dest/static/

function pull_artifact {
    job=$1
    path=$2
    buildname=$3
    rm $dest/files/tumelune-v$version-$buildname || true
    wget -c "$project_root/-/jobs/artifacts/v$version/raw/target/$path?job=$job" -O $dest/files/tumelune-v$version-$buildname
}

if [[ $PULL_ARTIFACTS ]]; then
    pull_artifact "build-windows-x86_64" "x86_64-pc-windows-gnu/release/tumelune.exe"   "windows-x64.exe"
    pull_artifact "build-linux-x86_64"   "release/tumelune"                             "linux-x64.bin"
    #pull_artifact "build-osx-x86_64"     "x86_64-apple-darwin/release/tumelune"         "osx-x64.bin"
    pull_artifact "build-windows-i686"   "i686-pc-windows-gnu/release/tumelune.exe"     "windows-x32.exe"
    pull_artifact "build-linux-i686"     "release/tumelune"                             "linux-x32.bin"
    rm $dest/files/tumelune-v$version-source.zip
    wget -c "$project_root/-/archive/v$version/tumelune-main.zip" -N -O $dest/files/tumelune-v$version-source.zip || exit 1
fi

url="file://$(realpath "$dest/index.html")"

echo
echo "$url"
xdg-open "$url"
