use lazy_static::lazy_static;

macro_rules! schema_struct {
    ( $name:ident with $($key:ident: $type:ident($options:expr), )* ) => {
        schema_struct!($name with $( $key: $type ($options) ),* );
    };
    ( $name:ident with $($key:ident: $type:ident($options:expr) ),* ) => {
        pub struct $name { $(pub $key: tantivy::schema::Field),*, pub schema: tantivy::schema::Schema }
        impl $name { paste::paste! {
            pub fn build() -> Self {
                use tantivy::schema::*; // important for $options
                let mut schema_builder = Schema::builder();
                Self {
                    $(
                        $key: schema_builder.[<add_ $type _field>](std::stringify!($key), $options),
                    )*
                    schema: schema_builder.build(),
                }
            }
        } }
    };
}

// those cool constants don't fit in a 32 bit isize
// it would be a shame not to write 'mastodon' on those eight bytes
#[repr(u64)]
#[derive(PartialEq, Copy, Clone)]
#[allow(non_camel_case_types)]
pub enum source_type {
    MASTODON = 0x6d61_7374_6f64_6f6e,
    TWITTER = 0x0074_7769_7474_6572,
}
impl source_type {
    pub fn from_u64(n: u64) -> Option<Self> {
        match n {
            0x6d61_7374_6f64_6f6e => Some(Self::MASTODON),
            0x0074_7769_7474_6572 => Some(Self::TWITTER),
            _ => None
        }
    }
}

// pub mod item_type {
//     pub const POST: u64 = 0;
//     pub const BOOST: u64 = 1;
// }

#[repr(u64)]
#[derive(PartialEq, Copy, Clone)]
#[allow(non_camel_case_types)]
pub enum item_type {
    POST = 0,
    BOOST = 1,
}

schema_struct! { TlSchema with
    id: u64(INDEXED | FAST | STORED),
    url: text(STRING | STORED),
    author: text(TEXT | STORED),
    summary: text(TEXT | STORED),
    body: text(TEXT | STORED),
    time: date(INDEXED | STORED),
    timef: f64(INDEXED | STORED | FAST),
    timeu: u64(INDEXED | STORED | FAST),
    attachment_count: u64(INDEXED | STORED | FAST),
    attachment_types: text(TEXT),
    attachments: text(TEXT | STORED),
    item_type: u64(INDEXED | FAST | STORED),
    source_type: u64(INDEXED | FAST | STORED),
    source: text(STRING | STORED),
    privacy: text(STRING | STORED),
    flags: text(TEXT | STORED),
}

lazy_static! {
    pub static ref SCHEMA: TlSchema = TlSchema::build();
}
