use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Clone, Copy, Debug)]
pub struct DailyPoint {
    pub time: u32,
    pub twitter: u32,
    pub mastodon: u32,
}

#[derive(Clone, Copy, Debug)]
pub enum Sort {
    Time(bool),
    Media(bool),
}
impl Sort {
    pub fn as_str(&self) -> &'static str {
        match self {
            Sort::Time(false) => "time",
            Sort::Time(true) => "time_asc",
            Sort::Media(false) => "media",
            Sort::Media(true) => "media_asc",
        }
    }
    pub fn from_str(s: &str) -> Self {
        match s {
            "media_asc" => Sort::Media(true),
            "media" => Sort::Media(false),
            "time_asc" => Sort::Time(true),
            "time" | "" | _ => Sort::Time(false),
        }
    }
}
