use anyhow::Result;
use chrono::{Datelike, Timelike};

use crate::db::DatabaseInserter;

pub fn timef(t: &chrono::NaiveDateTime) -> f64 {
    let int_part = t.year() * 1_00_00 + t.month() as i32 * 1_00 + t.day() as i32;
    let dec_part = t.hour() * 1_00_00 + t.minute() * 1_00 + t.second();

    int_part as f64 + dec_part as f64 / 1_00_00_00.0
}

pub fn timeu(t: &chrono::NaiveDateTime) -> u64 {
    let ymd = t.year() * 1_00_00 + t.month() as i32 * 1_00 + t.day() as i32;
    let hms = t.hour() * 1_00_00 + t.minute() * 1_00 + t.second();
    let mil = t.nanosecond() / 1_000_000;

    ymd as u64 * 1_000000_000 + hms as u64 * 1_000 + mil as u64
}

pub fn chrono_to_tantivy(t: &chrono::DateTime<chrono::Utc>) -> tantivy::DateTime {
    let unix_ts = t.timestamp_micros();
    // whos going to point at this variable and say 'its me'
    // i am Not an unix gal

    tantivy::DateTime::from_timestamp_micros(unix_ts)
}

pub trait DocIterator {
    fn source_type(&self) -> &'static str;
    fn process(&mut self, db: &mut DatabaseInserter, f: &mut dyn FnMut(f32)) -> Result<()>;
}

pub type BoxedDocIterator = Box<dyn DocIterator>;
use std::path::Path;

pub trait Archive {
    fn new(path: &Path) -> BoxedDocIterator;
}

#[derive(Debug, Clone, Copy, structopt::StructOpt)]
pub enum InputType {
    Detect,
    Mastodon,
    Twitter,
}
impl std::str::FromStr for InputType {
    type Err = &'static str;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s {
            "mastodon" => Ok(InputType::Mastodon),
            "twitter" => Ok(InputType::Twitter),
            "detect" => Ok(InputType::Detect),
            _ => Err("unknown input type value"),
        }
    }
}
impl std::fmt::Display for InputType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}",
            match self {
                Self::Detect => "detect",
                Self::Mastodon => "mastodon",
                Self::Twitter => "twitter",
            }
        )
    }
}
impl Default for InputType {
    fn default() -> Self {
        Self::Detect
    }
}

pub type Settings = std::collections::HashMap<String, String>;


pub type ImportResult = Result<BoxedDocIterator>;

pub trait AccountImporter {
    /// returns a map of settings keys to descriptors
    fn get_settings() -> Settings;

    /// synchronizes posts
    fn pull(settings: std::collections::HashMap<String, String>) -> ImportResult;
}
