use std::collections::HashMap;
use anyhow::{anyhow, Context};
use super::common::{AccountImporter, DocIterator, Settings, ImportResult};
use crate::db::DatabaseInserter;

pub struct MastodonAPI {
    auth: String,
    page_url: String,
}

fn query_as<T: serde::de::DeserializeOwned>(url: &str, auth: &str) -> anyhow::Result<T> {
    Ok(ureq::get(url)
        .set("Authorization", auth)
        .set("Accept", "application/activity+json")
        .call()?
        .into_json()?)
}

impl AccountImporter for MastodonAPI {
    fn get_settings() -> Settings {
        return [
            ("".to_string(), "".to_string()),
        ].into();
    }

    fn pull(settings: HashMap<String, String>) -> ImportResult {
        let instance_url = settings.get("instance_url").ok_or(anyhow!("instance_url missing"))?;
        let account_name = settings.get("account_name").ok_or(anyhow!("account_name missing"))?;
        let bearer = settings.get("bearer").ok_or(anyhow!("bearer missing"))?;

        let auth = "Bearer ".to_owned() + bearer;
        let outbox_url = format!("{}/api/v1/users/{}/outbox", instance_url, account_name);
        let outbox = query_as(&outbox_url, &auth)
            .context("failed to query outbox")?;

        log::info!("outbox: {:?}", outbox);
        let page_url = "";

        // pass activitypub accept header to get paging?
        // o no it's about hitting outbox instead of statuses

        Ok(Box::new(MastodonAPI {
            page_url: page_url.to_string(),
            auth: auth,
        }))
    }
}

impl DocIterator for MastodonAPI {
    fn source_type(&self) -> &'static str {
        "mastodon"
    }
    fn process(&mut self, db: &mut DatabaseInserter, f: &mut dyn FnMut(f32)) -> anyhow::Result<()> {
        let mut url = self.page_url.clone();
        loop {
            //let page: OrderedCollectionPage = self.query_as(url.as_str());
            //log::info!("page: {:?}", page);
            break;
        }
        Ok(())
    }
}