function build_histogram(histogram, v) {
    const start_time = performance.now();
    var ctx = histogram.getContext('2d');
    const wid = 720; //histogram.offsetWidth;
    const hei = 80; //histogram.offsetHeight;
    const top_margin = 12;
    const bottom_margin = 12;
    histogram.width = wid;
    histogram.height = hei;
    // overwrite the css too - otherwise it can still get resized
    // and scaling errors ensue, like when the scrollbar is added
    histogram.style.width = wid + "px";
    histogram.style.height = hei + "px";

    const mid = Math.floor((hei - top_margin - bottom_margin) / 2);

    console.log('histogram:', histogram, `@ ${wid}x${hei}px mid ${mid}px`);

    // ooo center bar
    ctx.fillStyle = "#888";
    ctx.fillRect(0, mid-1 + top_margin, wid, 1);

    // find min/max times (& max value for y scaling)
    let min = 0, max = 0;
    let count_max = 1;
    for (let i=0; i<v.length; i++) {
        const row = v[i];
        const count = row.twitter + row.mastodon;
        if (min == 0 || min > row.time) min = row.time;
        if (max == 0 || max < row.time) max = row.time;
        if (count > count_max) count_max = count;
    }
    min = f_to_date(min);
    max = f_to_date(max);

    // ready to iterate over the Days
    // v can have gaps (count=0 are skipped) and we want to show those gaps
    // the idæ is to convert back/from Date to iterate over the calendar
    // could i do math over min/max? prolly but calendars are a mess
    let full_days = 0;
    for (let day=new Date(min); day<max; day.setDate(day.getDate() + 1)) {
        let day_f = date_to_f(day);
        full_days += 1;
    }
    
    let map = {};
    for (let i=0; i<v.length; i++) {
        const row = v[i];
        map[row.time] = {t: row.twitter, m: row.mastodon};
    }

    let divider = 1;
    let bar_width;
    if (full_days > wid) {
        // more than a day per bar. we'll need a divider
        divider = Math.ceil(full_days / wid);
    } else {
        // less than a day per bar. so one day per bar and variable width bars
    }

    bar_width = Math.floor((wid * divider) / full_days);
    console.log('histogram: span', full_days, 'days', `div=${divider}px bar_wid=${bar_width}px`);

    const cs = getComputedStyle(histogram);
    const yearMarker = cs.getPropertyValue("--histogram-year-marker");
    const monthMarker = cs.getPropertyValue("--histogram-month-marker"); // "#444";
    const textColor = cs.getPropertyValue("--histogram-text");
    const cursorTextColor = cs.getPropertyValue("--histogram-cursor-text");
    const background = cs.getPropertyValue("--background");

    // let's Plot oh Ye we're Plottin Finally
    const DAY = 3600 * 24;
    const x_base = Math.ceil(((wid / bar_width) - (full_days / divider)) / 2);
    function draw_bar(t, bar_n, highlight=false) {
        const day_f = date_to_f(t);

        const x_offset = x_base + bar_width * bar_n;

        // light background for months
        if (day_f % 100 <= divider) {
            if ((day_f % 10000 / 100) < 2) {
                // years
                ctx.fillStyle = background;
                ctx.fillRect(x_offset + 4, 0, 30, 10);
                ctx.fillStyle = textColor;
                ctx.font = "11px 'Iosevka Term Slab'";
                const year = Math.round(day_f / 10000);
                ctx.fillText(year, x_offset + 4, 10, 30);

                ctx.fillStyle = yearMarker;
                ctx.fillRect(x_offset, 0, 1, hei - bottom_margin);
            } else {
                // months

                // with text when there is no year text
                if (full_days < 365) {
                    ctx.fillStyle = background;
                    ctx.fillRect(x_offset + 4, 0, 20, 10);
                    ctx.fillStyle = textColor;
                    ctx.font = "11px 'Iosevka Term Slab'";
                    const month = (Math.floor(day_f / 100) % 100).toString().padStart(2, '0');
                    ctx.fillText(month, x_offset + 4, 10, 20);
                }

                ctx.fillStyle = monthMarker;
                ctx.fillRect(x_offset, top_margin, 1, (hei - top_margin - bottom_margin) / 2 - 1);
                ctx.fillRect(x_offset, (hei - top_margin - bottom_margin) / 2 + top_margin, 1, (hei - top_margin - bottom_margin) / 2 - 1);
            }
        }

        let colors = {t: "#996666", m: "#666699"}
        if (highlight) {
            colors = {t: "#ee8888", m: "#8888ee"}
        }
        const parts = {t: 0, m: 0};
        for (let i=0; i<divider; i++) {
            const data = map[day_f + (i * DAY)] || {t: 0, m: 0};
            parts.t += data.t || 0;
            parts.m += data.m || 0;
        }
        const sum = parts.t + parts.m;

        function ground(f) { return isFinite(f) ? f : 0; }
        function elevate(n) { return n == 0 ? 0 : n + 10; }
        const m = 0.10;

        const bar_h = (hei - top_margin - bottom_margin) - 1; // -1 for the center line
        const side_h = bar_h / 2;

        // compute bar sizes
        let bars = new Array();
        for (const p of Object.entries(parts).sort((a, b) => parts[a] - parts[b])) {
            const count = p[1];
            const h = Math.ceil((ground(Math.log(elevate(count)*m)) / Math.log((count_max + 1)*m)) * side_h);
            bars.push({hei: h, color: colors[p[0]]});
        }

        // draw each data bar
        const full_bar_hei = bars.map(a => a.hei).reduce((a, b) => a + b, 0);
        const y_margin = Math.round(side_h - full_bar_hei) - 1;
        let y_offset = y_margin;
        for (const part of bars) {
            if (part.hei == 0) { continue; }
            ctx.fillStyle = part.color;
            ctx.fillRect(x_offset, top_margin + y_margin, bar_width, part.hei);
            ctx.fillRect(x_offset, top_margin + y_margin + 1 + part.hei, bar_width, part.hei);
            y_offset += part.hei;
        }
    }

    function draw_bars() {
        let day = new Date(min);
        let bar_n = 0;
        while (day < max) {
            draw_bar(day, bar_n, false);

            bar_n += 1;
            day.setDate(day.getDate() + divider);
        }
    }
    draw_bars();

    function replace_timef(q, min, max) {
        const f = "timef:[" + date_to_f(min) + " TO " + date_to_f(max) + "]";

        q = q.trim();
        q = q.replace(/ *timef:\[[0-9 TO]+\] */, function(match, offset, string) {
            const idx = string.indexOf(match);
            let replacement = f;
            if (idx > 0) { replacement = " " + replacement; }
            if (idx < (string.length - match.length)) { replacement = replacement + " "; }
            return replacement;
        });
        q = q.trim();
        if (q.indexOf(f) === -1) {
            if (q.length > 0) { q += " "; }
            q += f; 
        }
        q = q.trim();
        return q;
    }

    let last_focus = null;
    let mouse_down_point = null;
    histogram.onmousemove = function (e) {
        const rect = this.getBoundingClientRect();
        const x = e.clientX - rect.left;
        const y = e.clientY - rect.top;

        const bar_n = Math.floor((x - x_base) / bar_width);

        if (last_focus !== null && mouse_down_point == null) {
            draw_bar(last_focus.t, last_focus.n, false);
        }

        if (bar_n < 0 || (bar_n * divider) > full_days) {
            return;
        }

        const t = plus_days(min, (bar_n * divider));

        // when we're dragging we actually want to highlight all bars between each point
        if (mouse_down_point != null) {
            draw_bars();

            let times = [t, mouse_down_point.t];
            let ns = [bar_n, mouse_down_point.n];
            let min_t = new Date(Math.min.apply(null, times));
            let max_t = plus_days(Math.max.apply(null, times), divider);
            let min_n = Math.min.apply(null, ns);
            let max_n = Math.max.apply(null, ns);
            const invert = (min_t < t) ? 1 : -1;
            const bar_delta = max_n - min_n;
            for (let tt=min_t, n=min_n; tt<max_t; tt=plus_days(tt, divider), n++) {
                draw_bar(tt, n, true);
            }
        }

        // draw the currently highlighted bar
        draw_bar(t, bar_n, true);
        last_focus = {t: t, n: bar_n};

        // and display the equivalent time
        ctx.clearRect(0, hei - bottom_margin, wid, bottom_margin);
        // draw text
        if (!t || isNaN(t)) return;
        ctx.fillStyle = cursorTextColor;
        ctx.font = "bold 10px 'Iosevka Term Slab'";
        let time_str = format_date(t, 'short');
        const ts = ctx.measureText(time_str);
        const adjx = Math.min(x, wid - ts.width - 10);
        ctx.fillText(time_str, adjx, hei - 3);
    };
    histogram.onmousedown = function (e) {
        mouse_down_point = last_focus;
    };
    histogram.onmouseup = function (e) {
        if (!mouse_down_point) return;
        const up_point = last_focus;
        let times = [up_point.t, mouse_down_point.t];
        let min = new Date(Math.min.apply(null, times));
        let max = plus_days(Math.max.apply(null, times), divider);

        if (!isFinite(min) || !isFinite(max)) {
            return;
        }

        const query_field = document.getElementById("query_field");
        query_field.value = replace_timef(query_field.value, min, max);

        draw_bars();
        mouse_down_point = null;
    };

    const end_time = performance.now();
    console.log("histogram: finished in", end_time - start_time, "ms");

    // haha Im absolutely not focusing on that to avoid thinking about my issues
}
