if (!Array.prototype.last){
    Array.prototype.last = function(){
        return this[this.length - 1];
    };
};

function format_date(d, fmt='') {
    if (d === null || d === 0) return "<null>"
    if (d instanceof Date) {
        d = d.toISOString();
    }
    d = d.replace("T", " ");
    d = d.replace("+00:00", " UTC");
    d = d.replace(".000Z", " UTC");

    if (fmt == 'short') {
        d = d.replace(' 00:00:00 UTC', '');
        // i think this is for DST? i dont feel like bothering
        d = d.replace(' 01:00:00 UTC', '');
    }
    return d;
}
function format_timef(f, format='full') {
    const year = Math.floor(f / 10000).toString().padStart(4, '0');
    const month = Math.floor((f / 100) % 100).toString().padStart(2, '0');
    const day = Math.floor(f % 100).toString().padStart(2, '0');
    const h = Math.floor(f % 1 * 100).toString().padStart(2, '0');
    const m = Math.floor(f * 10000 % 100).toString().padStart(2, '0');
    const s = Math.floor(f * 1000000 % 100).toString().padStart(2, '0');
    if (format == 'date') {
        return `${year}-${month}-${day}`;
    }
    if (format == 'time') {
        return `${h}:${m}:${s}`;
    }
    return `${year}-${month}-${day} ${h}:${m}:${s}`;
}

// convert between Date and the integer part of a 'timef' field.
// (those are YYYYMMDD.HHMMSS floats)
function f_to_date(f, time=false) {
    const year = Math.floor(f / 10000).toString().padStart(4, '0');
    const month = Math.floor((f / 100) % 100).toString().padStart(2, '0');
    const day = Math.floor(f % 100).toString().padStart(2, '0');
    const hour = Math.floor((f * 100) % 100).toString().padStart(2, '0');
    const mins = Math.floor((f * 10000) % 100).toString().padStart(2, '0');
    const secs = Math.floor((f * 1000000) % 100).toString().padStart(2, '0');
    let timepart = "00:00:00";
    if (time) {
        timepart = `${hour}:${mins}:${secs}`;
    }
    return new Date(`${year}-${month}-${day}T${timepart}Z`);
}
function date_to_f(d) {
    return (
        d.getFullYear() * 10000 +
        (d.getMonth() + 1) * 100 + // haha yeah months are 0-indexed. so mucg fun
        d.getDate()
    );
}

function plus_days(d, n) {
    let o = new Date(d);
    o.setDate(o.getDate() + n);
    return o;
}

function atag(href, text, onclick) {
    const t = document.createElement("a");
    t.href = href;
    t.innerText = text;
    if (onclick) t.onclick = onclick;
    //if (onclick) t.onclick = function() { onclick(); return false; };
    return t;
}

function tdtag(value) {
    const t = document.createElement("td");
    t.innerHTML = value;
    return t;
}

function format_timespan(rows) {
    let min = null, max = null;
    for (let i=0; i<rows.length; i++) {
        const row = rows[i];
        if (min == null || min > row.timef) min = row.timef;
        if (max == null || max < row.timef) max = row.timef;
    }
    const delta = max - min;

    let text;
    if (delta > 2) { // more than 48h
        text = `from ${format_timef(min, 'date')} to ${format_timef(max, 'date')}`;
    }
    else if (delta > 1) { // more than 24h
        text = `from ${format_timef(min, 'date')} to ${format_timef(max, 'date')}`;
    }
    else { // less than 24h
        text = `from ${format_timef(min)} to ${format_timef(max, 'time')}`;
    }

    min = f_to_date(min, true);
    max = f_to_date(max, true);
    if (delta >= 2) {
        text += ` (${Math.round((max-min) / (3600 * 24 * 1000))} days)`;
    } else if (delta > 0.01) {
        text += ` (${Math.round((max-min) / (3600 * 1000))} hours)`;
    }
    return text;
}


function el(tag, v) {
    const e = document.createElement(tag);
    for (let i=1; i<arguments.length; i++) {
        const v = arguments[i];
        if (typeof v === "function") v(e);
        if (typeof v === "string" || v instanceof String) e.innerText = v;
        if (v instanceof HTMLElement) e.append(v);
    }
    return e;
}

function format_bytesize(n) {
    const units = [
        {u: 'T', m: 1e12},
        {u: 'G', m: 1e9},
        {u: 'M', m: 1e6},
        {u: 'K', m: 1e3},
    ];
    for (const u of units) {
        if (n >= u.m) {
            return `${(n / u.m).toFixed(1)} ${u.u}B`;
        }
    }
    return `${n} Bytes`;
}