

class TumeluneInterface {
}

class TlTauri extends TumeluneInterface {
    getVersion() {
        return window.__TAURI__.app.getVersion();
    }
    getStatus() {
        return window.__TAURI__.invoke('get_status');
    }
    getResults(args) {
        return window.__TAURI__.invoke('get_results', args);
    }
    getCounts(args) {
        return window.__TAURI__.invoke('get_counts', args);
    }
    clearImportQueue(args) {
        return window.__TAURI__.invoke('clear_import_queue');
    }
    clearDatabase(args) {
        return window.__TAURI__.invoke('clear_database');
    }
    importFile(args) {
        window.__TAURI__.invoke('import_file', {path: args.path || args.filename });
    }
    openFileDialog() {
        return window.__TAURI__.dialog.open({
                directory: false,
                multiple: true,
                filters: [{name: 'Archives (zip, tgz, tar)', extensions: ['zip', 'tgz', 'gz', 'tar']}],
            });
    }
}
class TlFetch extends TumeluneInterface {
    getVersion() {
        // thers no dedicated endpoint so we just wrap a call to /status
        return fetch('/status').then((v) => v.json()).then((v) => v.version);
    }
    getStatus() {
        return fetch('/status').then((v) => v.json());
    }
    getResults(args) {
        return fetch("/search?q=" + encodeURIComponent(args.q)+`&max=${args.max}&offset=${args.offset}&sort=${args.sort}`, {
            method: 'POST',
        }).then((v) => v.json());
    }
    getCounts(args) {
        return fetch("/count?q=" + encodeURIComponent(args.q), {
            method: 'POST',
        }).then((v) => v.json());
    }
    clearImportQueue(args) {
        return fetch("/import_queue", {
            method: 'DELETE',
        });
    }
    clearDatabase(args) {
        return fetch("/clear", {
            method: 'POST',
        });
    }
    importFile(args) {
        const formData = new FormData();
        formData.append('file', args.path || args.file);
        return fetch("/import", {
            method: 'POST',
            body: formData,
        });
    }
    openFileDialog() {
        const input = document.createElement('input');
        input.type = 'file';
        input.click();
        return new Promise((resolve) => {
            input.onchange = () => {
                const value = input.files[0];
                if (!value) return resolve();
                console.log(value);
                resolve(value);
            };
        });
    }
}

var tumelune_api;
if (window.__TAURI__) {
    tumelune_api = new TlTauri();
} else {
    tumelune_api = new TlFetch();
}
