function build_results_header(pageheader, response, query_results) {
    const status_text = document.createElement("p");
    pageheader.appendChild(status_text);

    const new_results = response.results;

    const timespan = format_timespan(new_results);
    const page_s = (response.n_pages > 0) ? `, page ${response.page+1}/${response.n_pages+1}` : "";
    status_text.innerText = `displaying ${new_results.length}/${response.total} results${page_s} ${timespan}`;
    status_text.classList.add("status-text");

    const sortbuttonp = document.createElement("p");
    const sortbutton = document.createElement("select");
    pageheader.appendChild(sortbuttonp);
    sortbuttonp.appendChild(sortbutton);
    sortbuttonp.classList.add("sort-button");
    sortbutton.appendChild(new Option("time >", "time"));
    sortbutton.appendChild(new Option("time <", "time_asc"));
    sortbutton.appendChild(new Option("media >", "media"));
    sortbutton.appendChild(new Option("media <", "media_asc"));
    sortbutton.onchange = function() {
        query_results(response.query, response.offset, response.page_size, this.value);
    };
    sortbutton.value = response.sort;

    const pagebuttons = document.createElement("p");
    pagebuttons.classList.add("page-buttons");
    pageheader.appendChild(pagebuttons);

    if (response.page > 1) {
        pagebuttons.append(el("a", a => {
            a.href = "#";
            a.title = "First page";
            a.innerText = "1";
            a.onclick = function() {
                query_results(response.query, 0, response.page_size, response.sort);
                return false;
            };
        }));
    }
    if (response.page > 0) {
        pagebuttons.append(el("a", a => {
            a.href = "#";
            a.title = "Previous page";
            a.classList.add("button");
            a.innerText = "<";
            a.onclick = function() {
                query_results(response.query, response.offset - response.page_size, response.page_size, response.sort);
                return false;
            };
        }));
    }
    pagebuttons.appendChild(el("span", (response.page + 1).toString()));
    if (response.page < response.n_pages) {
        pagebuttons.append(el("a", a => {
            a.href = "#";
            a.title = "Next page";
            a.classList.add("button");
            a.innerText = ">";
            a.onclick = function() {
                query_results(response.query, response.offset + response.page_size, response.page_size, response.sort);
                return false;
            };
        }));
    }
    if (response.page < response.n_pages && response.n_pages >= 2) {
        pagebuttons.append(el("a", a => {
            a.href = "#";
            a.title = "Last page";
            a.innerText = (response.n_pages + 1).toString();
            a.onclick = function() {
                query_results(response.query, (response.n_pages) * response.page_size, response.page_size, response.sort);
                return false;
            };
        }));
    }
}

function build_error_header(pageheader, msg) {
    const status_text = document.createElement("p");
    pageheader.appendChild(status_text);
    status_text.innerText = "error: " + msg;
    status_text.classList.add("status-text");
}

function view_media(media) {
    const d = document.getElementById("media_view_d");

    const c = d.querySelector(".content");
    c.innerHTML = "";
    c.appendChild(media);
    c.style.textAlign = "center";

    show_dialog("media_view_d");
}

function preparePath(path) {
    if (path.startsWith('/attachment'))
        path = path.substr(11);

    return `http://localhost:29804/attachment/${path}`;
    return window.__TAURI__.tauri.convertFileSrc(path, 'attachments');
    return `attachments://localhost/${path}`;
}


function video_thumbnail(attrs) {
    const box = document.createElement("div");
    box.classList.add("video-thumbnail");

    const video_tag = document.createElement("video");
    if (attrs.title)
        video_tag.title = attrs.title;
    video_tag.title = attrs.title;
    video_tag.controls = false;
    video_tag.preload = "auto";
    video_tag.onerror = (e) => {
        console.error("failed to load video: ", e);
    };
    const source = document.createElement('source');
    source.src = attrs.src;
    if (attrs.data) {
        source.type = attrs.data.mime;
    }
    video_tag.append(source);

    const link = document.createElement("a");
    link.href = attrs.src;
    link.appendChild(video_tag);
    link.onclick = function(e) {
        e.preventDefault();

        const full = document.createElement("video");
        if (attrs.title)
            full.title = attrs.title;
        full.controls = true;

        const source = document.createElement('source');
        source.src = attrs.src;
        if (attrs.data) {
            source.type = attrs.data.mime;
        }
        full.appendChild(source);

        view_media(full);

        full.onloadedmetadata = function() {
            const overlay = document.getElementById("overlay");
            const maxSize = Math.min(overlay.offsetWidth, overlay.offsetHeight) - 40;
            console.log(full);
            full.style.maxWidth = Math.min(full.videoWidth, maxSize) + "px";
            full.style.maxHeight = Math.min(full.videoHeight, maxSize) + "px";
        };
    };
    box.appendChild(link);
    return box;
}

function image_thumbnails(params) {
    const img = document.createElement("img");
    img.title = params.title;
    img.src = params.url;
    const link = document.createElement("a");
    link.appendChild(img);
    link.href = img.src;
    link.onclick = function(e) {
        e.preventDefault();

        const full = document.createElement("img");
        full.title = params.name;
        full.src = params.url;
        view_media(full);


        const overlay = document.getElementById("overlay");
        const maxSize = Math.min(overlay.offsetWidth, overlay.offsetHeight) - 40;
        full.style.maxWidth = Math.min(full.width, maxSize) + "px";
        full.style.maxHeight = Math.min(full.height, maxSize) + "px";
    }
    return link;
}

function build_mastodon_attachments(thumbnails, result) {
    if (!result.source.attachment) return;

    for (let i=0; i<result.source.attachment.length; i++) {
        const att = result.source.attachment[i];
        const clean_url = att.url.replace(/^\/+/, '');
        if (att.mediaType.startsWith("image")) {
            thumbnails.appendChild(image_thumbnails({
                url: preparePath(`${result.source_type}/${clean_url}?media_type=${att.mediaType}`),
                title: att.name,
            }));
        }
        else if (att.mediaType.startsWith("video")) {
            thumbnails.appendChild(video_thumbnail({
                title: att.name,
                data: {
                    source: result.source_type,
                    path: clean_url,
                    mime: att.mediaType
                },
                src: preparePath(`${result.source_type}/${clean_url}?media_type=${att.mediaType}`),
                type: att.mediaType
            }));
        }
        else if (att.mediaType.startsWith("audio")) {
            const atte = document.createElement("audio");
            atte.controls = true;
            atte.title = att.name;
            atte.src = preparePath('mastodon/' + clean_url);
            thumbnails.appendChild(atte);
        }
    }
}

function build_twitter_attachments(thumbnails, result) {
    const entities = result.source.extended_entities || result.source.entities;
    if (!entities || !entities.media) return;

    for (let i=0; i<entities.media.length; i++) {
        const att = entities.media[i];
        if (att.type == "photo" || att.type == undefined) {
            const filename = att.media_url.split('/').last()
            thumbnails.appendChild(image_thumbnails({
                url: `/attachment/${result.source_type}/tweet_media/${result.source.id_str}-${filename}`,
                title: att.name,
            }))
        } else if (att.type == "video" || att.type == "animated_gif") {
            const variants = att.video_info.variants.filter(x => x.bitrate !== undefined);
            variants.sort((x, y) => x.bitrate > y.bitrate);
            const filename = variants[0].url.split('/').last();
            thumbnails.appendChild(video_thumbnail({
                title: att.name,
                src: `/attachment/${result.source_type}/tweet_media/${result.source.id_str}-${filename}`
            }));
        }
    }
}

function build_mastodon_poll(items, mode) {
    const pollUl = document.createElement("table");
    pollUl.classList.add("poll");
    const total = Array.from(items).map(e => e["replies"]["totalItems"]).reduce((a, n) => a + n, 0);

    const bullet = (mode == "one") ? "○" : "□";

    for (const item of items) {
        const count = item["replies"]["totalItems"];
        const optLi = document.createElement("li");
        const percent = (count/total*100).toFixed(0);
        //:optLi.classList.add("");

        optLi.append(el("div", e => {
            e.classList.add("bar");
            e.style.width = `${percent}%`;
        }));


        const inner = el("div");
        inner.classList.add("inner");
        optLi.append(inner);

        inner.append(el("span", bullet, e => e.classList.add("bullet")));
        inner.append(el("span", e => {
            e.classList.add("count");
            e.innerText = `${count}/${total}`;
        }));
        inner.append(el("span", e => {
            e.classList.add("count");
            e.innerText = `${percent}%`;
        }));
        inner.append(el("span", e => {
            e.classList.add("text");
            e.innerText = item["name"];
        }));
        pollUl.append(optLi);
    }
    return pollUl;
}

function status_visibility_icon(result) {
    switch (result.privacy) {
        case "public": return "🌐";
        case "unlisted": return "🔓";
        case "followers": return "🔒";
        case "dm": return "✉️";
    }
    return "?";
}

function build_entry_header(header_div, body_div, result) {
    const expand = document.createElement("a");
    expand.href = "#";
    expand.innerText = "+";
    expand.classList.add("expand");
    expand.onclick = function() {
        if (body_div.style.display == "none") {
            expand.innerText = "–";
            body_div.style.display = "block";
        } else {
            expand.innerText = "+";
            body_div.style.display = "none";
        }
        return false;
    };
    header_div.appendChild(expand);

    const display = document.createElement("div");
    display.classList.add("display");
    header_div.appendChild(display);

    const line = document.createElement("div");
    line.classList.add("line");
    {
        if (result.privacy != null) {
            const vis = document.createElement("span");
            vis.innerText = status_visibility_icon(result);
            vis.title = result.privacy;
            vis.classList.add("visibility")
            line.appendChild(vis);
        }
        
        const url = document.createElement("a");
        url.href = result.url;
        url.target = "_blank";
        url.innerText = result.url;
        line.appendChild(url);

        const att = document.createElement("p");
        att.classList.add("attachment-count");
        att.innerText = (function(c) {
            if (c > 1) return `${c} attachments`;
            if (c == 1) return `${c} attachment`;
            return "";
        })(result.attachment_count);
        line.appendChild(att);

        const date = document.createElement("p");
        date.classList.add("date");
        date.innerText = format_date(result.time);
        line.appendChild(date);
    }
    display.appendChild(line);

    const content = document.createElement("div");
    let text_content = result.body;
    if (result.summary) {
        text_content = "<i>[cw] " + result.summary + "</i><br />" + text_content;
    }
    content.innerHTML = text_content;
    content.classList.add("content");
    display.appendChild(content);

    // ensure links have target=_blank so we dont replace the whole webview
    content.querySelectorAll("a").forEach(a => {
        a.target = "_blank";
    });

    if (result.source.oneOf) {
        display.appendChild(build_mastodon_poll(result.source.oneOf, "one"));
    }
    if (result.source.anyOf) {
        display.appendChild(build_mastodon_poll(result.source.anyOf, "any"));
    }

    if (result.attachment_count > 0) {
        const thumbnails = document.createElement("div");
        thumbnails.classList.add("attachment-thumbnails");

        if (result.source_type == "mastodon") {
            build_mastodon_attachments(thumbnails, result);
        }
        else if (result.source_type == "twitter") {
            build_twitter_attachments(thumbnails, result);
        }
        else {
            console.log(result);
        }
        display.appendChild(thumbnails);
    }
}

function build_entry_body(body_div, result) {
    const meta = Object.assign({}, result);
    delete meta.body; delete meta.source;
    const content = (
        "/* Source: */\n" +
        JSON.stringify(result.source, null, 2) +
        "\n/* tumelune metadata: */\n" +
        JSON.stringify(meta, null, 2)
    );
    const pre = document.createElement("pre");
    const code = document.createElement("code");
    code.classList.add("json");
    code.innerText = content;
    pre.appendChild(code);
    body_div.appendChild(pre);

    hljs.highlightBlock(code);
}

window.addEventListener("load", function() {
    const results = document.querySelector(".results");
    const header = results.querySelector(".header");
    const query_form = document.querySelector("form.query");
    const query_field = document.querySelector("input[type=text]");

    const body = document.body;
    const theme = window.localStorage.getItem('tumelune_theme');
    if (theme) {
        for (const v of body.classList.values()) {
            if (v.startsWith('theme-')) {
                body.classList.remove(v);
            }
        }
        body.classList.add('theme-' + theme);
    }

    function update_results(response) {
        results.innerHTML = "";

        const pageheader = document.createElement("div");
        results.appendChild(pageheader);
        pageheader.classList.add("pageheader");

        if (response.error) {
            build_error_header(pageheader, response.error);
            return;
        }

        build_results_header(pageheader, response, query_results);

        const new_results = response.results;
        for (var i=0; i<new_results.length; i++) {
            const result = new_results[i];
            const result_div = document.createElement("div");

            const header_div = document.createElement("div");
            header_div.classList = "header";
            const body_div = document.createElement("div");
            body_div.classList = "body";
            body_div.style.display = "none";
            result_div.appendChild(header_div);
            result_div.appendChild(body_div);

            build_entry_header(header_div, body_div, result);
            build_entry_body(body_div, result);

            results.appendChild(result_div);
        }

        const pagefooter = document.createElement("div");
        results.appendChild(pagefooter);
        pagefooter.classList.add("pageheader");
        build_results_header(pagefooter, response, query_results);
    }

    var last_query, last_offset, last_max, last_sort;
    function query_results(query, offset=0, max=null, sort="-time") {
        last_query = query;
        last_offset = offset;
        last_max = max;
        last_sort = sort;

        if (max === null)
            max = settings.page_entries.get('page_entries');

        tumelune_api.getResults({
            q: query,
            max: max,
            offset: offset,
            sort: sort,
        }).then((results) => {
            update_results(results);
        });

        query_histogram();
    }

    tumelune_api.getVersion().then((v) => {
        document.getElementById('tumelune_version').innerText = v;
    });

    tumelune_api.getStatus().then((status) => {
        document.getElementById("changelog").innerHTML = status.changelog;
    });

    const histogram = document.getElementById("timeline");
    function query_histogram() {
        tumelune_api.getCounts({q: last_query}).then((counts) => {
            build_histogram(histogram, counts);
        });
    }

    query_form.onsubmit = (ev) => {
        query_results(query_field.value)
        return false;
    };
    query_results("");

    document.getElementById("refresh").onclick = function() {
        query_results(query_field.value, last_offset, last_max, last_sort);
    };
    document.getElementById("reset").onclick = function() {
        query_field.value = "";
        query_results("");
    };

});